@@ -1,5 +1,7 @@
 /* A more useful interface to strtol.
-   Copyright (C) 1995, 1996, 1998-2000 Free Software Foundation, Inc.
+
+   Copyright (C) 1995, 1996, 1998, 1999, 2000, 2001, 2003 Free
+   Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,42 +33,20 @@
    need stderr defined if assertion checking is enabled.  */
 #include <stdio.h>
 
-#if STDC_HEADERS
-# include <stdlib.h>
-#endif
-
-#if HAVE_STRING_H
-# include <string.h>
-#else
-# include <strings.h>
-# ifndef strchr
-#  define strchr index
-# endif
-#endif
-
 #include <assert.h>
 #include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
 
 #include <errno.h>
 #ifndef errno
 extern int errno;
 #endif
 
-#if HAVE_LIMITS_H
-# include <limits.h>
-#endif
-
-#ifndef CHAR_BIT
-# define CHAR_BIT 8
-#endif
+#include <limits.h>
 
 /* The extra casts work around common compiler bugs.  */
 #define TYPE_SIGNED(t) (! ((t) 0 < (t) -1))
-/* The outer cast is needed to work around a bug in Cray C 5.0.3.0.
-   It is necessary at least when t == time_t.  */
-#define TYPE_MINIMUM(t) ((t) (TYPE_SIGNED (t) \
-			      ? ~ (t) 0 << (sizeof (t) * CHAR_BIT - 1) : (t) 0))
-#define TYPE_MAXIMUM(t) (~ (t) 0 - TYPE_MINIMUM (t))
 
 #if defined (STDC_HEADERS) || (!defined (isascii) && !defined (HAVE_ISASCII))
 # define IN_CTYPE_DOMAIN(c) 1
@@ -78,19 +58,11 @@ extern int errno;
 
 #include "xstrtol.h"
 
-#ifndef strtol
-long int strtol ();
-#endif
-
-#ifndef strtoul
-unsigned long int strtoul ();
-#endif
-
-#ifndef strtoimax
+#if !HAVE_DECL_STRTOIMAX && !defined strtoimax
 intmax_t strtoimax ();
 #endif
 
-#ifndef strtoumax
+#if !HAVE_DECL_STRTOUMAX && !defined strtoumax
 uintmax_t strtoumax ();
 #endif
 
@@ -141,8 +113,16 @@ __xstrtol (const char *s, char **ptr, int strtol_base,
   tmp = __strtol (s, p, strtol_base);
   if (errno != 0)
     return LONGINT_OVERFLOW;
+
   if (*p == s)
-    return LONGINT_INVALID;
+    {
+      /* If there is no number but there is a valid suffix, assume the
+	 number is 1.  The string is invalid otherwise.  */
+      if (valid_suffixes && **p && strchr (valid_suffixes, **p))
+	tmp = 1;
+      else
+	return LONGINT_INVALID;
+    }
 
   /* Let valid_suffixes == NULL mean `allow any suffix'.  */
   /* FIXME: update all callers except the ones that allow suffixes
@@ -169,15 +149,20 @@ __xstrtol (const char *s, char **ptr, int strtol_base,
 	{
 	  /* The ``valid suffix'' '0' is a special flag meaning that
 	     an optional second suffix is allowed, which can change
-	     the base, e.g. "100MD" for 100 megabytes decimal.  */
+	     the base.  A suffix "B" (e.g. "100MB") stands for a power
+	     of 1000, whereas a suffix "iB" (e.g. "100MiB") stands for
+	     a power of 1024.  If no suffix (e.g. "100M"), assume
+	     power-of-1024.  */
 
 	  switch (p[0][1])
 	    {
-	    case 'B':
-	      suffixes++;
+	    case 'i':
+	      if (p[0][2] == 'B')
+		suffixes += 2;
 	      break;
 
-	    case 'D':
+	    case 'B':
+	    case 'D': /* 'D' is obsolescent */
 	      base = 1000;
 	      suffixes++;
 	      break;
@@ -198,29 +183,30 @@ __xstrtol (const char *s, char **ptr, int strtol_base,
 	  overflow = 0;
 	  break;
 
-	case 'E': /* Exa */
+	case 'E': /* exa or exbi */
 	  overflow = bkm_scale_by_power (&tmp, base, 6);
 	  break;
 
-	case 'G': /* Giga */
+	case 'G': /* giga or gibi */
 	case 'g': /* 'g' is undocumented; for compatibility only */
 	  overflow = bkm_scale_by_power (&tmp, base, 3);
 	  break;
 
 	case 'k': /* kilo */
+	case 'K': /* kibi */
 	  overflow = bkm_scale_by_power (&tmp, base, 1);
 	  break;
 
-	case 'M': /* Mega */
+	case 'M': /* mega or mebi */
 	case 'm': /* 'm' is undocumented; for compatibility only */
 	  overflow = bkm_scale_by_power (&tmp, base, 2);
 	  break;
 
-	case 'P': /* Peta */
+	case 'P': /* peta or pebi */
 	  overflow = bkm_scale_by_power (&tmp, base, 5);
 	  break;
 
-	case 'T': /* Tera */
+	case 'T': /* tera or tebi */
 	case 't': /* 't' is undocumented; for compatibility only */
 	  overflow = bkm_scale_by_power (&tmp, base, 4);
 	  break;
@@ -229,11 +215,11 @@ __xstrtol (const char *s, char **ptr, int strtol_base,
 	  overflow = bkm_scale (&tmp, 2);
 	  break;
 
-	case 'Y': /* Yotta */
+	case 'Y': /* yotta or 2**80 */
 	  overflow = bkm_scale_by_power (&tmp, base, 8);
 	  break;
 
-	case 'Z': /* Zetta */
+	case 'Z': /* zetta or 2**70 */
 	  overflow = bkm_scale_by_power (&tmp, base, 7);
 	  break;
 
@@ -261,7 +247,7 @@ __xstrtol (const char *s, char **ptr, int strtol_base,
 char *program_name;
 
 int
-main (int argc, char** argv)
+main (int argc, char **argv)
 {
   strtol_error s_err;
   int i;
